/*
 * Copyright (C) 2010 Google Inc. All rights reserved.
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"

#if USE(ACCELERATED_COMPOSITING)

#include "ImageLayerWiiU.h"

#include "Image.h"

namespace WebCore {

PassRefPtr<ImageLayerWiiU> ImageLayerWiiU::create(GraphicsLayerWiiU* owner)
{
    return adoptRef(new ImageLayerWiiU(owner));
}

ImageLayerWiiU::ImageLayerWiiU(GraphicsLayerWiiU* owner)
    : LayerWiiU(owner)
{
}

ImageLayerWiiU::~ImageLayerWiiU()
{
}

void ImageLayerWiiU::cleanupResources()
{
}

void ImageLayerWiiU::setContents(Image* contents)
{
    // setContents() currently gets called whenever there is any
    // style change that affects the layer even if that change doesn't
    // affect the actual contents of the image (e.g. a CSS animation).
    // With this check in place we avoid unecessary texture uploads.
    if ((m_contents == contents))
        return;

    m_contents = contents;
    m_dirtyRect = IntRect(IntPoint(0, 0), bounds());
    setNeedsDisplay();
}

void ImageLayerWiiU::paintContents(GraphicsContext *ctx)
{
    if (m_contents) {
        ctx->drawImage(m_contents.get(), ColorSpaceDeviceRGB, m_owner->contentsRect(), m_contents->rect());        
        
        //Rather than modify the image, we draw the debug borders overtop.
        //This is less efficient, but we are debugging anyway...
#ifndef NDEBUG 
        if (m_owner->showDebugBorders()) {
            drawDebugBorder(ctx, m_owner->contentsRect().width(), m_owner->contentsRect().height());
        }
#endif        
    }
}

bool ImageLayerWiiU::drawsContent() const
{
    return m_contents;
}

}
#endif // USE(ACCELERATED_COMPOSITING)
