/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
//////////////////////////////////////////////////////////////////////////
//	Name	: MediaPlayerPrivateUnix.h
//
//	Purpose	: This is the unix WiiU version of the MediaPlayerPrivate
//				interface
//////////////////////////////////////////////////////////////////////////
#include "config.h" //This goes first to define all of the proper ENABLEs and things
#include <fstream>
#include <dlfcn.h>
#include <stdarg.h>
#include "TimeRanges.h"
#include "MediaPlayerPrivateUnix.h"
#include "GraphicsContext.h"
#include "GraphicsContextPlatformPrivate.h"
extern "C" {
#include "libshim/include/shim_win32.h"
}

using namespace WebCore;

bool g_EnableMediaMessageLogging = true;
extern WiiURenderer * g_MainRenderer;

namespace WebCore
{
};	//End WebCore



//////////////////////////////////////////////////////////////////////////
//	Name	: LogMediaMessage
//////////////////////////////////////////////////////////////////////////
void LogMediaMessage(const char * vMessage, ...)
{
	int          FormattedMessageLength;
	char         ErrorBuffer[512];
	char *       FormattedMessage;
	va_list      VariableArgList;
	std::fstream OutFile;

	//If there was no message given, do nothing
	if (!vMessage)
		return;

	// Todo
}

void MediaPlayerPrivateUnixTimerObject::UpdateVideo(WebCore::Timer<MediaPlayerPrivateUnixTimerObject>* vTimer)
{
	m_pOwner->m_pOwner->repaint();

    #if USE(ACCELERATED_COMPOSITING)
		if( m_videoLayer.get() && supportsAcceleratedRendering() )
			m_videoLayer->setNeedsDisplay( IntRect( 0, 0, m_videoLayer->bounds().width(), m_videoLayer->bounds().height()) );
#endif
	if (m_pOwner->m_HasPlaybackStateChanged)
	{
		m_pOwner->m_pOwner->playbackStateChanged();
		m_pOwner->m_HasPlaybackStateChanged = false;
	}
}


//////////////////////////////////////////////////////////////////////////
//	Name	: mimeTypeCache (Lifted straight from MediaPlayerPrivateWiiU)
//////////////////////////////////////////////////////////////////////////
static HashSet<String> mimeTypeCache()
{
	static HashSet<String> TypeCache;
	static bool InitializedCache = false;

	if(!InitializedCache)
	{
		TypeCache.add(String("application/vnd.apple.mpegurl"));	// HLS
		TypeCache.add(String("audio/aac"));						// AAC
		TypeCache.add(String("audio/ogg"));						// Ogg Vorbis
		TypeCache.add(String("video/mp4"));						// H.264 & possibly AAC
		TypeCache.add(String("video/mp2t"));					// MP2TS

		TypeCache.add(String("video/avi"));						// AVI, because why not.

		InitializedCache = true;
	}

	return TypeCache;
}


//////////////////////////////////////////////////////////////////////////
//	Name	: getSupportedTypes
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::getSupportedTypes(HashSet<String>& vTypes)
{
	vTypes = mimeTypeCache();
}


//////////////////////////////////////////////////////////////////////////
//	Name	: supportsType
//////////////////////////////////////////////////////////////////////////
MediaPlayer::SupportsType MediaPlayerPrivateUnix::supportsType(const String& vType, const String& vCodecs)
{
	//We support ALL the things!
	return MediaPlayer::IsSupported;
}


//////////////////////////////////////////////////////////////////////////
//	Name	: registerMediaEngine
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::registerMediaEngine(MediaEngineRegistrar vRegistrar)
{
	vRegistrar(create, getSupportedTypes, supportsType, 0, 0, 0);
}


//////////////////////////////////////////////////////////////////////////
//	Name	: create
//////////////////////////////////////////////////////////////////////////
PassOwnPtr<MediaPlayerPrivateInterface> MediaPlayerPrivateUnix::create(MediaPlayer* player)
{
	return adoptPtr(new MediaPlayerPrivateUnix(player));
}


//////////////////////////////////////////////////////////////////////////
//	Name	: Constructor
//////////////////////////////////////////////////////////////////////////
// MediaPlayerPrivateUnix::MediaPlayerPrivateUnix() : m_pMediaDLL(0), m_pMediaPlayerShim(0), m_IsVisible(true), m_VideoTextureID(GAMEPAD_UNDEFINED_ID)
// {
// 	//TODO -- MEDIA: This DLL needs to be copied during the build process over
// 	//					to the same location as the DLL. This function should
// 	//					load the DLL relative to the EXE.
//
// 	//Load the media DLL
// 	LoadMediaDLL("X:/VS2010/MediaFramework/Release/MediaFramework.dll");
// }


//////////////////////////////////////////////////////////////////////////
//	Name	: Constructor
//////////////////////////////////////////////////////////////////////////
MediaPlayerPrivateUnix::MediaPlayerPrivateUnix(MediaPlayer * vOwner) : m_pMediaDLL(0), m_pMediaPlayerShim(0), m_IsVisible(true), m_VideoTextureID(GAMEPAD_UNDEFINED_ID), m_HasPlaybackStateChanged(false)
{
	char DLLFilePath[2048];

	m_pOwner = vOwner;

	//Set up our update timer
	m_UpdateTimer.m_pOwner = this;
	m_UpdateTimer.m_Timer.startRepeating(60.0 / 1000.0f);

	//Load the media DLL
	snprintf(DLLFilePath, sizeof(DLLFilePath), "%s\\%s", shim_GetBaseDir(), "assets/MediaFramework.so");
	LoadMediaDLL(DLLFilePath);
}


//////////////////////////////////////////////////////////////////////////
//	Name	: Destructor
//////////////////////////////////////////////////////////////////////////
MediaPlayerPrivateUnix::~MediaPlayerPrivateUnix()
{
	//Make sure we're not playing anything
	if (m_pMediaPlayerShim)
		m_pMediaPlayerShim->Shutdown();

	//If we haven't deleted our video texture, do that now
	if (m_VideoTextureID != GAMEPAD_UNDEFINED_ID)
		DeleteOpenGLTexture(m_VideoTextureID);

	//Unload the media DLL and free used memory
	UnloadMediaDLL();
}


//////////////////////////////////////////////////////////////////////////
//	Name	: load
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::load(const String& vURL)
{
	MPERROR Result;

	//Make sure we were given a valid URL
	if (!vURL.ascii().data())
	{
		LOG_SIMPLE_MEDIA_MSG("MediaPlayerPrivateUnix::load() called with a null URL\n");
		return;
	}
	LOG_MEDIA_MSG("MediaPlayerPrivateUnix::load(\"%s\")\n", vURL.ascii().data());

	//Open the given URL
	Result = m_pMediaPlayerShim->OpenURL(vURL.ascii().data());
	if (Result)
		LOG_MEDIA_MSG("    Encountered error: %s\n", MPERRORToString(Result));

	m_pOwner->networkStateChanged();
	m_pOwner->readyStateChanged();
}


//////////////////////////////////////////////////////////////////////////
//	Name	: cancelLoad
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::cancelLoad()
{
	LOG_SIMPLE_MEDIA_MSG("MediaPlayerPrivateUnix::cancelLoad()\n");
}


//////////////////////////////////////////////////////////////////////////
//	Name	: play
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::play()
{
	MPERROR Result;
	LOG_SIMPLE_MEDIA_MSG("MediaPlayerPrivateUnix::play()\n");

	//Play
	Result = m_pMediaPlayerShim->Play();
	if (Result)
		LOG_MEDIA_MSG("    Encountered error: %s\n", MPERRORToString(Result));

	m_HasPlaybackStateChanged = true;
}


//////////////////////////////////////////////////////////////////////////
//	Name	: pause
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::pause()
{
	MPERROR Result;
	LOG_SIMPLE_MEDIA_MSG("MediaPlayerPrivateUnix::pause()\n");

	//Pause
	Result = m_pMediaPlayerShim->Pause();
	if (Result)
		LOG_MEDIA_MSG("    Encountered error: %s\n", MPERRORToString(Result));

	m_HasPlaybackStateChanged = true;
}


//////////////////////////////////////////////////////////////////////////
//	Name	: seek
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::seek(float vTime)
{
	MPERROR Result;
	LOG_MEDIA_MSG("MediaPlayerPrivateUnix::seek(%4.4f)\n", vTime);

	//Pause
	Result = m_pMediaPlayerShim->SeekToTime(vTime);
	if (Result)
		LOG_MEDIA_MSG("    Encountered error: %s\n", MPERRORToString(Result));
}


//////////////////////////////////////////////////////////////////////////
//	Name	: setVolume
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::setVolume(float vVolume)
{
	MPERROR Result;
	LOG_MEDIA_MSG("MediaPlayerPrivateUnix::setVolume(%4.4f)\n", vVolume);

	//Set the volume as desired
	Result = m_pMediaPlayerShim->SetVolume(vVolume);
	if (Result)
		LOG_MEDIA_MSG("    Encountered error: %s\n", MPERRORToString(Result));
}


//////////////////////////////////////////////////////////////////////////
//	Name	: naturalSize
//////////////////////////////////////////////////////////////////////////
IntSize MediaPlayerPrivateUnix::naturalSize() const
{
	tPoint2D VideoSize = m_pMediaPlayerShim->GetVideoSize();

	return IntSize(VideoSize.x, VideoSize.y);
}


//////////////////////////////////////////////////////////////////////////
//	Name	: buffered
//////////////////////////////////////////////////////////////////////////
PassRefPtr<TimeRanges> MediaPlayerPrivateUnix::buffered() const
{
	//TODO -- MEDIA: When the WIIU version supports this, we'll need to as well.

	RefPtr<TimeRanges> RetVal = TimeRanges::create();
    return RetVal.release();
}


//////////////////////////////////////////////////////////////////////////
//	Name	: paint
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::paint(GraphicsContext * vContext, const IntRect& vDest)
{
	unsigned int Width;
	unsigned int Height;

	if (!m_pMediaPlayerShim)
		return;

	// Please see comment at method: MediaPlayerPrivateWiiU::naturalSize()
// 	if (vContext->paintingDisabled() || !hasVideo())
// 		return;
	Width = m_pMediaPlayerShim->GetVideoSize().x;
	Height = m_pMediaPlayerShim->GetVideoSize().y;

	//If we don't have any video image data, just draw a black box
	if (!m_pMediaPlayerShim->GetVideoImageBuffer())
	{
		Color black(0,0,0,255);
		vContext->fillRect(FloatRect(vDest), black, ColorSpaceDeviceRGB);
		return;
	}

	//If we don't have texture data for this video, go ahead and create it
	if (m_VideoTextureID == GAMEPAD_UNDEFINED_ID)
	{
		GAMEPADERROR Result;
		m_VideoTextureID = CreateOpenGLColorBufferTexture(Result, Width, Height);
		printf("Created video texture: %d x %d, result: %d. New texture ID: %d", Width, Height, Result, m_VideoTextureID);
	}

	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, m_VideoTextureID);
	glTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, Width, Height, GL_RGBA, GL_UNSIGNED_BYTE, m_pMediaPlayerShim->GetVideoImageBuffer());
	glDisable(GL_TEXTURE_2D);

	vContext->platformContext()->makeCurrent();
	g_MainRenderer->DrawImage(m_VideoTextureID, vDest.x(), vDest.y(), vDest.x() + vDest.width(), vDest.y() + vDest.height(), 0, 0, 1, 1, 1.0f);
}
#if USE(ACCELERATED_COMPOSITING)
//-----------------------------------------------------------------------------
/**
*/ 
// whether accelerated rendering is supported by the media engine for the current media.
bool MediaPlayerPrivateUnix::supportsAcceleratedRendering() const
{
    return true;
}

//-----------------------------------------------------------------------------
/**
*/ 
// called when the rendering system flips the into or out of accelerated rendering mode.
void MediaPlayerPrivateUnix::acceleratedRenderingStateChanged()
{ 
    setUpVideoRendering(); 
} 

bool MediaPlayerPrivateUnix::isReadyForVideoSetup() const
{
    return m_readyState >= MediaPlayer::HaveMetadata && m_pOwner->visible();
}

MediaPlayerPrivateUnix::MediaRenderingMode MediaPlayerPrivateUnix::currentRenderingMode() const
{
    if (platformLayer())
       return MediaRenderingToLayer;

    if (m_pOwner->mediaPlayerClient())
        return MediaRenderingToContext;

    return MediaRenderingNone;
}

MediaPlayerPrivateUnix::MediaRenderingMode MediaPlayerPrivateUnix::preferredRenderingMode() const
{
    if (!m_pOwner->visible() || !m_pOwner->frameView())
        return MediaRenderingNone;

    if (supportsAcceleratedRendering() && m_pOwner->mediaPlayerClient()->mediaPlayerRenderingCanBeAccelerated(m_pOwner))
         return MediaRenderingToLayer;

    return MediaRenderingToContext;
}

void MediaPlayerPrivateUnix::setUpVideoRendering()
{
    if (!isReadyForVideoSetup())
        return;

    MediaRenderingMode currentMode = currentRenderingMode();
    MediaRenderingMode preferredMode = preferredRenderingMode();
    if (currentMode == preferredMode && currentMode != MediaRenderingNone)
       return;

    if (currentMode != MediaRenderingNone) {  
        tearDownVideoRendering();
    }
    if (preferredMode == MediaRenderingToLayer) {
        createVideoLayer();   
    }

    // If using a movie layer, inform the client so the compositing tree is updated.
    if (currentMode == MediaRenderingToLayer || preferredMode == MediaRenderingToLayer) {
          m_pOwner->mediaPlayerClient()->mediaPlayerRenderingModeChanged(m_pOwner);
   }

}

void MediaPlayerPrivateUnix::createVideoLayer()
{
    if (m_videoLayer) {
        return;
    }
     m_videoLayer = VideoLayerWiiU::create(0); 
     m_videoLayer->setPlayer(m_pOwner); 
}

void MediaPlayerPrivateUnix::tearDownVideoRendering()
{
    if (platformLayer()) {
        m_videoLayer.release(); 
    }
}
#endif

//////////////////////////////////////////////////////////////////////////
//	Name	: LoadMediaDLL
//////////////////////////////////////////////////////////////////////////
MPERROR MediaPlayerPrivateUnix::LoadMediaDLL(const char * vDLLName)
{
	MPERROR Result;
	tMFVSGCCShim * (*GetMediaPlayerShim)() = 0;

	//Make sure we haven't already loaded the media DLL
	if (m_pMediaDLL)
	{
		ASSERT(false && "Attempting to load media DLL when it is already loaded.");
		return MPERROR_SUCCESS;
	}

	//Load the media DLL
	m_pMediaDLL = dlopen(vDLLName, RTLD_NOW);
	if (!m_pMediaDLL)
		return MPERROR_CANNOT_LOAD_DLL;

	//Find the function to get the media player
	GetMediaPlayerShim = (tMFVSGCCShim * (*)())dlsym(m_pMediaDLL, "GetMediaPlayerShim");
	if (!GetMediaPlayerShim)
	{
		dlclose(m_pMediaDLL);
		m_pMediaDLL = NULL;
		return MPERROR_INVALID_MEDIA_DLL;
	}

	//Get the media player from the DLL
	m_pMediaPlayerShim = GetMediaPlayerShim();
	if (!m_pMediaPlayerShim)
	{
		dlclose(m_pMediaDLL);
		m_pMediaDLL = NULL;
		return MPERROR_FAILURE;
	}

	//Initialize the media player
	IMediaPlayer::tInitParams InitParams;
	Result = m_pMediaPlayerShim->Initialize(InitParams);
	if (Result)
	{
// 		char WHYDWEUNLOAD[512] = {0};
// 		sprintf(WHYDWEUNLOAD, "Initialization failed because %u", Result);
// 		MessageBoxA(0, WHYDWEUNLOAD, "Error", 0);

		UnloadMediaDLL();
		return Result;
	}

	//All went well, return success;
	return MPERROR_SUCCESS;
}


//////////////////////////////////////////////////////////////////////////
//	Name	: UnloadMediaDLL
//////////////////////////////////////////////////////////////////////////
void MediaPlayerPrivateUnix::UnloadMediaDLL()
{
	void (*FreeMediaPlayerShim)(tMFVSGCCShim *);

	//If the DLL isn't loaded, we're done
	if (!m_pMediaDLL)
		return;

	//Free used memory
	FreeMediaPlayerShim = (void (*)(tMFVSGCCShim *))dlsym(m_pMediaDLL, "FreeMediaPlayerShim");
	if (FreeMediaPlayerShim)
		FreeMediaPlayerShim(m_pMediaPlayerShim);

	//Close the DLL
	dlclose(m_pMediaDLL);
	m_pMediaDLL = NULL;
	m_pMediaPlayerShim = 0;
}
