/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions 
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright 
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the 
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY 
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY 
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; 
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "DataTransferItemWiiU.h"

#if ENABLE(DATA_TRANSFER_ITEMS)

#include "Blob.h"
#include "Clipboard.h"
#include "NotImplemented.h"
#include "StringCallback.h"

namespace WebCore {

PassRefPtr<DataTransferItem> DataTransferItem::create(PassRefPtr<Clipboard> owner,
                                                      ScriptExecutionContext* context,
                                                      const String& data,
                                                      const String& type)
{
    return DataTransferItemWiiU::create(owner, context, type, data);
}

PassRefPtr<DataTransferItemWiiU> DataTransferItemWiiU::createFromPasteboard(PassRefPtr<Clipboard> owner,
                                                                        ScriptExecutionContext* context,
                                                                        const String& type)
{
    if (type == "text/plain" || type == "text/html")
        return adoptRef(new DataTransferItemWiiU(owner, context, PasteboardSource, DataTransferItem::kindString, type, ""));

    return adoptRef(new DataTransferItemWiiU(owner, context, PasteboardSource, DataTransferItem::kindFile, type, ""));
}

PassRefPtr<DataTransferItemWiiU> DataTransferItemWiiU::create(PassRefPtr<Clipboard> owner,
                                                          ScriptExecutionContext* context,
                                                          const String& type,
                                                          const String& data)
{
    return adoptRef(new DataTransferItemWiiU(owner, context, InternalSource, DataTransferItem::kindString, type, data));
}

DataTransferItemWiiU::DataTransferItemWiiU(PassRefPtr<Clipboard> owner,
                                       ScriptExecutionContext* context,
                                       DataSource source,
                                       const String& kind, const String& type,
                                       const String& data)
    : DataTransferItem(owner, kind, type)
    , m_context(context)
    , m_dataSource(source)
    , m_data(data)
{
}

void DataTransferItemWiiU::getAsString(PassRefPtr<StringCallback> callback)
{
    if ((owner()->policy() != ClipboardReadable && owner()->policy() != ClipboardWritable)
        || kind() != kindString)
        return;

    if (m_dataSource == InternalSource) {
        callback->scheduleCallback(m_context, m_data);
        return;
    }

    return;
}

PassRefPtr<Blob> DataTransferItemWiiU::getAsFile()
{
    notImplemented();
    return 0;
}

}

#endif // ENABLE(DATA_TRANSFER_ITEMS)
