/*
 *  Copyright (C) 2007 Holger Hans Peter Freyther zecke@selfish.org
 *            (C) 2009 Kenneth Rohde Christiansen
 *            (C) 2009 INdT, Instituto Nokia de Technologia
 *            (C) 2009-2010 ProFUSION embedded systems
 *            (C) 2009-2010 Samsung Electronics
 *            (c) 2012 NINTENDO.
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include "config.h"
#include "ScrollbarWiiU.h"

#include "ChromeClient.h"
#include "Frame.h"
#include "FrameView.h"
#include "GraphicsContext.h"
#include "HostWindow.h"
#include "IntRect.h"
#include "NotImplemented.h"
#include "Page.h"
#include "ScrollbarTheme.h"

#include <string>
#include <wtf/text/CString.h>

using namespace std;
using namespace WebCore;

PassRefPtr<Scrollbar> Scrollbar::createNativeScrollbar(ScrollableArea *area, ScrollbarOrientation orientation, ScrollbarControlSize size)
{
    return adoptRef(new ScrollbarWiiU(area, orientation, size));
}

ScrollbarWiiU::ScrollbarWiiU(ScrollableArea *area, ScrollbarOrientation orientation, ScrollbarControlSize size)
    : Scrollbar(area, orientation, size)
    , m_lastPos(0)
    , m_lastTotalSize(0)
    , m_lastVisibleSize(0)
{
    #if PLATFORM(WIIU)
    Widget::setFrameRect(IntRect(0, 0, 0, 0));
    #else
    Widget::setFrameRect(IntRect(0, 0, 16, 16));
    #endif
}

ScrollbarWiiU::~ScrollbarWiiU()
{
}

void ScrollbarWiiU::setParent(ScrollView* view)
{
    Widget::setParent(view);
}

void ScrollbarWiiU::updateThumbPosition()
{
    updateThumbPositionAndProportion();
}

void ScrollbarWiiU::updateThumbProportion()
{
    updateThumbPositionAndProportion();
}

void ScrollbarWiiU::updateThumbPositionAndProportion()
{
    #if !PLATFORM(WIIU)
    Scrollbar::updateThumbProportion();
    #endif
}

void ScrollbarWiiU::setFrameRect(const IntRect& rect)
{
    Widget::setFrameRect(rect);
    frameRectsChanged();
}

void ScrollbarWiiU::frameRectsChanged()
{
    #if !PLATFORM(WIIU)
    Scrollbar::frameRectsChanged();
    #endif
}

void ScrollbarWiiU::paint(GraphicsContext* context, const IntRect& rect)
{
    #if !PLATFORM(WIIU)
    //How the heck does the WIIU build work without this?
    Scrollbar::paint(context, rect);
    #endif
}

