/*
 * Copyright (c) 2012 NINTENDO.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY NINTENDO AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL NINTENDO OR CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef WIIUFOCUSANIMATOR_H
#define WIIUFOCUSANIMATOR_H

#include <Timer.h>
#include <FloatRect.h>
#include <IntRect.h>


namespace WebCore{
	class Node;
	class ChromeClient;
	class FloatRect;
	class Document;
	class HitTestResult;
	class HitTestRequest;
	class RenderView;
	class IntPoint;
}


class WiiUFocusAnimator {
public:
	WiiUFocusAnimator();
    virtual ~WiiUFocusAnimator();

	void startChangeFocusAnimation(WebCore::Node *to, bool skipToEnd = false);
	bool isChangingFocus(void) {return m_isChangingFocus;}
	bool isFocusNaved(void) {return m_focusNaved;}
	void setFocusNaved(bool focusNaved) {m_focusNaved = focusNaved;}
	void setLastEventWasMouseEvent(bool lastEventMouseEvent);
    bool getLastEventWasMouseEvent() { return m_lastEventMouseEvent; }
	WebCore::FloatRect getCurrentFocusRect();
	void setDocument(WebCore::Document *doc);
	void FrameClosing();
	void redrawWindow();
	void startFadeFocusAnimation();
    WebCore::Node* getCurrentFocusNode() {return m_curFocusNode;}

private:
	void updateChangeFocusRect(WebCore::Timer<WiiUFocusAnimator>*);
	void updateFadeFocusRect(WebCore::Timer<WiiUFocusAnimator>*);
	void reset();
	WebCore::Node* getNodeFromPoint(int x, int y);
	bool isValidNode(WebCore::FloatRect rect);

	WebCore::Document *m_doc;

	WebCore::FloatRect m_startFocusRect;
	WebCore::FloatRect m_curFocusRect;
	WebCore::FloatRect m_endFocusRect;
	WebCore::Node* m_curFocusNode;

	WebCore::IntRect m_damageRect;

	WebCore::Timer<WiiUFocusAnimator> m_focusChangeTimer;
	WebCore::Timer<WiiUFocusAnimator> m_focusFadeTimer;

	WebCore::ChromeClient *m_client;

	bool m_validToNode;
	bool m_validFromNode;

	float	m_stepX;
	float 	m_stepY;
	float 	m_stepW;
	float 	m_stepH;

	float   m_lastScrollX;
	float   m_lastScrollY;

	int		m_numIters;

	bool m_isChangingFocus;
	bool m_focusNaved;
	bool m_lastEventMouseEvent;
    bool m_focusRectIsInflated;
    bool m_isFadingFocus;
};

#endif
