#ifndef PLATFORM_ANIMATION_H
#define PLATFORM_ANIMATION_H

/**
	This namespace is for managing system level animations exposed via nwf.ui.Animation
*/
#include <string>
#include <vector>

#include "wiiu/shim_memMgr.h"

namespace LibPlatform 
{
	namespace Animation 
	{

		enum AnimationState {
			ANIMATION_RUNNING,
			ANIMATION_STOPPING,
			ANIMATION_FINISHED,
			ANIMATION_NUM_STATES
		};

		enum DisplayType {
			DISPLAY_TV,
			DISPLAY_GAMEPAD,
			NUM_DISPLAY_TYPES
		};

		struct AnimationInfo {
			unsigned animationAssetID;
			bool premultipliedAlpha;
			int x;
			int y;
			int cellWidth;
			int cellHeight;
			int numFrames;
			bool looping;
			int loopStartFrame;
			int loopEndFrame;
			float framerate;
			int fadeDuration;
		};

		struct AnimationInstance {

			AnimationInstance() : instanceID(0), currentFrame(0), fadeFramesRemaining(0), state(ANIMATION_RUNNING), vertexMemory(0), texcoordMemory(0)
			{
				vertexMemory = static_cast<float*>(shim_allocFromHeap(SHIM_DEFAULT_HEAP, 3*3*sizeof(float), 0x40));
				texcoordMemory = static_cast<float*>(shim_allocFromHeap(SHIM_DEFAULT_HEAP, 4*2*sizeof(float), 0x40));
			}

			~AnimationInstance()
			{
				if(vertexMemory)
				{
					shim_freeToHeap(vertexMemory);
					vertexMemory = 0;
				}
				if(texcoordMemory)
				{
					shim_freeToHeap(texcoordMemory);
					texcoordMemory = 0;
				}
			}

			unsigned instanceID;
			float currentFrame; //we can increment frames by non-whole numbers.
			int fadeFramesRemaining; //for fading in/out
			LibPlatform::Animation::AnimationState state;
			LibPlatform::Animation::AnimationInfo info;
			float *vertexMemory;
			float *texcoordMemory;
		};

		void InitializeAnimations(unsigned displayWidth, unsigned displayHeight);

		//returns an animation asset ID - Should be called when a Javascript Animation object is created
		unsigned NotifyAnimationCreated(void *pNativeTexture);
		
		//should be called when a Javascript Animation object gets finalized
		void NotifyAnimationDestroyed(unsigned animationAssetID);
		
		//retuns an animation instance handle
		unsigned StartAnimation(AnimationInfo &params, DisplayType display);
		bool StopAnimation(unsigned animationInstanceHandle, unsigned fadeDuration, DisplayType display);
		bool StopAllAnimations(unsigned fadeDuration, DisplayType display);
		bool ForceCleanUpAllAnimations();
		void TranslateAnimation(unsigned animationInstanceHandle, int x, int y, DisplayType display);

		void UpdateAnimations(DisplayType display);
		unsigned GetNumAnimations(DisplayType display);
		AnimationInstance *GetAnimationInfo(DisplayType display, unsigned index);



	}
}

#endif //PLATFORM_ANIMATION_H
