
#ifndef PLATFORMUI_H
#define PLATFORMUI_H

#include <list>
#include <vector>
#include "PlatformConfig.h" 

#define TEXTBOX_MAX_LENGTH 2048

namespace LibPlatform {
	namespace UI {

        const int MAX_MENU_ITEMS = 500; 

		enum UIFileList
		{
			FONT_SHADER,
			LAYOUT_SHADER,
			UI_DIALOG_BASE,
			UI_DIALOG_PROMPT,
			UI_DIALOG_BUTTON,
			UI_DIALOG_TITLE,
			UI_SCROLLBAR_BUTTON,
			UI_DIALOG_TEXT_BUTTON,
			UI_DIALOG_AUTHENTICATION,
			UI_MEDIA_TOOLBAR,
			UI_DROPDOWN_LIST,
			UI_BUTTON_OK,
			UI_DROPDOWN_LIST_BUTTON,
			UI_BACKGROUND_MASK,
			TOTAL_FILES
		};

        struct  UICallbacks_t {
            void (*mediaPlayPause)(LibPlatform::DisplayType display);
            void (*mediaQuit) (LibPlatform::DisplayType display);
            void (*mediaScreenSize) (LibPlatform::DisplayType display); 
            void (*mediaDisplay) (LibPlatform::DisplayType display); 
            void (*mediaCurrentTime) ( LibPlatform::DisplayType display, float currentTime);
			void (*mediaAdvance)  (LibPlatform::DisplayType display, float time);
            void (*mediaPlaybackRate) (LibPlatform::DisplayType display, float rate); 
            void (*MenuCloseCallback)(bool ok, const std::list<int>& selectedOptions, void* param);
            void (*mediaShowControls)(bool isTv);
            void (*mediaHideControls)(bool isTv);
            void (*mediaSetHWUse)(bool usingHWKeys);
        }; 
        
		void Initialize(UICallbacks_t *callbacks);
		void Shutdown(void);
		
		struct MenuListItem {
			MenuListItem() {}
			MenuListItem( const std::wstring& lab, bool isOptGr, bool selected = false, bool disabled = false) {
				label = lab;
				isOptionGroup = isOptGr;
				isSelected = selected;
                isDisabled = disabled; 
			}

			std::wstring label;
			bool isOptionGroup;
			bool isSelected;
            bool isDisabled;
		};

		void RenderUI(void* _pColorBuffer);
		void RenderUIGamepad(void* _pColorBuffer);
		
		bool OpenPrompt(const wchar_t* msg, std::size_t msg_len, const wchar_t* default_result, std::size_t default_result_len, wchar_t** result, bool openOnGamePad = true);
		bool OpenConfirm(const wchar_t* msg, std::size_t msg_len, bool openOnGamePad = true);
        void OpenAlert(const wchar_t* msg, std::size_t msg_len, bool openOnGamePad = true);
		void OpenAlertNoScroll(const wchar_t* msg, std::size_t msg_len, bool openOnGamePad = true);
        bool OpenAuthenticationPrompt(const wchar_t *url, wchar_t **user, wchar_t **pass, bool openOnGamePad = true);
        void OpenMediaControls(bool isTv);
		void OpenMenuList(const std::vector<LibPlatform::UI::MenuListItem>&, bool multiple, void* selectElement, bool isTv, bool isRTLAlign, bool focus_act);
		
		void CloseAll(void);
		void ClosePrompt(void);
		void CloseConfirm(void);
        void CloseAlert(void);
		void CloseAlertNoScroll(void);
		void CloseAuthenticationPrompt(void);
		void CloseMediaControls(bool isTv);
		void CloseMenuList( bool closeWithOk, bool isTv );
        void ForceCloseMenuList(bool closeWithOk, bool isTv);
		
        void MediaTogglePlayPause(bool play, bool isTv); 
        void MediaToggleFullScreen(bool enterFullScreen, bool isTv); 
        void MediaShowHideControls(bool show, bool isTv); 
        void MediaSetTimeLine(float currentTime, float duration, float bufferedTime, bool isTv); 
        void MediaSetFullScreenValid(bool isValid, bool isTv); 
        void MediaSetLocalFullScreenValid(bool isValid, bool isTv); 
        void MediaSetChangeDisplayValid(bool isValid, bool isTv); 
		bool MediaGetControlsVisible(bool isTv); 
        bool MediaIsPlayingFullscreen(bool isTv);
        bool MediaHitsControls(bool isTv, float x, float y);
        void MediaSetLoadingIcon(bool isloading);
        bool MediaIsLoading();
        float MediaGetFade(bool isTv);
        void MediaSetFade(bool isTv, float fadeToBlack);
        bool MediaDoFade(bool isTv, int direction, double timeStep);

		bool DoSKBDCapture(void);
		void SWKBDCallback(const std::wstring& inputText);
		void SWKBDCancelCallback();

		bool MenuListIsOpen(bool isTv);
        bool IsShowingOnTV(void);
        bool IsShowingOnGP(void);
        bool IsRemoteCursorDisabled(void);
        bool IsGamePadCursorDisabled(void);

		bool IsFocusNavActive(void);
		void SetFocusNavActive(bool isFocusNav);

		void OpenGamepadNotification();
		void CloseGamepadNotification();
		bool IsGamePadNotificationOpen();
		void SetMenuListAlertOpen(bool open);
	}
}

#endif
